import { ConnectionArguments, Connection } from 'graphql-relay';

type DeepReadonly<T> = T extends Record<string, unknown> | unknown[] | readonly unknown[] ? {
    readonly [P in keyof T]: DeepReadonly<T[P]>;
} : T;
/** Convert `{ a: number, b: string }` and `{ b: boolean, c: symbol }` into `{ a: number, b: boolean, c: symbol }`. */
type Merge<F, S> = {
    [K in keyof F | keyof S]: K extends keyof S ? S[K] extends Required<S>[K] ? S[K] : K extends keyof F ? Required<S>[K] | F[K] : S[K] : K extends keyof F ? F[K] : never;
};

type FieldResolverOptions<TypeWithTransientFields> = {
    seq: number;
    get: <FieldName extends keyof TypeWithTransientFields>(fieldName: FieldName) => Promise<DeepReadonly<TypeWithTransientFields[FieldName]> | undefined>;
};
declare class Dynamic<TypeWithTransientFields, Field> {
    private readonly factory;
    constructor(factory: (options: FieldResolverOptions<TypeWithTransientFields>) => Field | Promise<Field>);
    get(options: FieldResolverOptions<TypeWithTransientFields>): Promise<Field>;
}
/** Wrapper to delay field generation until needed. */
declare function dynamic<TypeWithTransientFields, Field>(factory: (options: FieldResolverOptions<TypeWithTransientFields>) => Field | Promise<Field>): Dynamic<TypeWithTransientFields, Field>;
type FieldResolver<TypeWithTransientFields, Field> = Field | Dynamic<TypeWithTransientFields, Field>;
/** The type of `defaultFields` or `inputFields` option. */
type FieldsResolver<TypeWithTransientFields> = {
    [FieldName in keyof TypeWithTransientFields]?: FieldResolver<TypeWithTransientFields, DeepReadonly<TypeWithTransientFields[FieldName]>>;
};
type ResolvedField<T extends FieldResolver<unknown, unknown>> = T extends FieldResolver<infer _, infer R> ? R : never;
/** Convert `{ a: number, b: () => number, c: () => Promise<number> }` into `{ a: number, b: number, c: number }`. */
type ResolvedFields<FieldsResolver extends Record<string, FieldResolver<unknown, unknown>>> = {
    [FieldName in keyof FieldsResolver]: ResolvedField<FieldsResolver[FieldName]>;
};

declare function resetSequence(key: object): void;
declare function resetAllSequence(): void;

type Traits<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> = {
    [traitName: string]: {
        defaultFields: FieldsResolver<Type & TransientFields>;
    };
};
interface TypeFactoryDefineOptions<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _Traits extends Traits<Type, TransientFields>> {
    defaultFields: _DefaultFieldsResolver;
    traits?: _Traits;
}
interface TypeFactoryInterface<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends Partial<Record<keyof Type, FieldResolver<Type & TransientFields, unknown>>>, _Traits extends Traits<Type, TransientFields>> {
    build(): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>>;
    build<T extends FieldsResolver<Type & TransientFields>>(inputFieldsResolver: T): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>>;
    buildList(count: number): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>[]>;
    buildList<T extends FieldsResolver<Type & TransientFields>>(count: number, inputFieldsResolver: T): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>[]>;
    buildConnection(count: number, connectionArgs: ConnectionArguments): Promise<Connection<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>>>;
    buildConnection<T extends FieldsResolver<Type & TransientFields>>(count: number, connectionArgs: ConnectionArguments, inputFieldsResolver: T): Promise<Connection<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>>>;
    use<T extends keyof _Traits>(traitName: T): TypeFactoryInterface<Type, TransientFields, Merge<_DefaultFieldsResolver, _Traits[T]['defaultFields']>, _Traits>;
    resetSequence(): void;
}
interface DefineTypeFactoryInterfaceRequired<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> {
    <_DefaultFieldsResolver extends Required<FieldsResolver<Type & TransientFields>>, _Traits extends Traits<Type, TransientFields>>(options: TypeFactoryDefineOptions<Type, TransientFields, _DefaultFieldsResolver, _Traits>): TypeFactoryInterface<Type, TransientFields, _DefaultFieldsResolver, _Traits>;
    withTransientFields<NewTransientFields extends Record<string, unknown>>(defaultTransientFields: NewTransientFields): DefineTypeFactoryInterface<Type, Merge<TransientFields, NewTransientFields>>;
    withAdditionalFields<AdditionalFields extends Record<string, unknown>>(): DefineTypeFactoryInterface<Type & AdditionalFields, TransientFields>;
}
interface DefineTypeFactoryInterface<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> extends DefineTypeFactoryInterfaceRequired<Type, TransientFields> {
    <_DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _Traits extends Traits<Type, TransientFields> = Traits<Type, TransientFields>>(options: TypeFactoryDefineOptions<Type, TransientFields, _DefaultFieldsResolver, _Traits>): TypeFactoryInterface<Type, TransientFields, _DefaultFieldsResolver, _Traits>;
}
declare const defineTypeFactory: DefineTypeFactoryInterfaceRequired<{}, {}>;

export { type DefineTypeFactoryInterface, type DefineTypeFactoryInterfaceRequired, defineTypeFactory, dynamic, resetAllSequence, resetSequence };
