import { Connection, ConnectionArguments } from 'graphql-relay';
import { ResolvedFields, FieldsResolver, FieldResolver } from './field-resolver.js';
import { Merge } from './util.js';
export type Traits<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> = {
    [traitName: string]: {
        defaultFields: FieldsResolver<Type & TransientFields>;
    };
};
export interface TypeFactoryDefineOptions<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _Traits extends Traits<Type, TransientFields>> {
    defaultFields: _DefaultFieldsResolver;
    traits?: _Traits;
}
export interface TypeFactoryInterface<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends Partial<Record<keyof Type, FieldResolver<Type & TransientFields, unknown>>>, _Traits extends Traits<Type, TransientFields>> {
    build(): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>>;
    build<T extends FieldsResolver<Type & TransientFields>>(inputFieldsResolver: T): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>>;
    buildList(count: number): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>[]>;
    buildList<T extends FieldsResolver<Type & TransientFields>>(count: number, inputFieldsResolver: T): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>[]>;
    buildConnection(count: number, connectionArgs: ConnectionArguments): Promise<Connection<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<{}>>, keyof TransientFields>>>;
    buildConnection<T extends FieldsResolver<Type & TransientFields>>(count: number, connectionArgs: ConnectionArguments, inputFieldsResolver: T): Promise<Connection<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<T>>, keyof TransientFields>>>;
    use<T extends keyof _Traits>(traitName: T): TypeFactoryInterface<Type, TransientFields, Merge<_DefaultFieldsResolver, _Traits[T]['defaultFields']>, _Traits>;
    resetSequence(): void;
}
export interface DefineTypeFactoryInterfaceRequired<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> {
    <_DefaultFieldsResolver extends Required<FieldsResolver<Type & TransientFields>>, _Traits extends Traits<Type, TransientFields>>(options: TypeFactoryDefineOptions<Type, TransientFields, _DefaultFieldsResolver, _Traits>): TypeFactoryInterface<Type, TransientFields, _DefaultFieldsResolver, _Traits>;
    withTransientFields<NewTransientFields extends Record<string, unknown>>(defaultTransientFields: NewTransientFields): DefineTypeFactoryInterface<Type, Merge<TransientFields, NewTransientFields>>;
    withAdditionalFields<AdditionalFields extends Record<string, unknown>>(): DefineTypeFactoryInterface<Type & AdditionalFields, TransientFields>;
}
export interface DefineTypeFactoryInterface<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>> extends DefineTypeFactoryInterfaceRequired<Type, TransientFields> {
    <_DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _Traits extends Traits<Type, TransientFields> = Traits<Type, TransientFields>>(options: TypeFactoryDefineOptions<Type, TransientFields, _DefaultFieldsResolver, _Traits>): TypeFactoryInterface<Type, TransientFields, _DefaultFieldsResolver, _Traits>;
}
export declare function defineTypeFactoryInternal<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _Traits extends Traits<Type, TransientFields>>(transientFieldNames: (keyof TransientFields)[], { defaultFields: defaultFieldsResolver, traits, }: TypeFactoryDefineOptions<Type, TransientFields, _DefaultFieldsResolver, _Traits>, seqKey?: object): TypeFactoryInterface<Type, TransientFields, _DefaultFieldsResolver, _Traits>;
export declare const defineTypeFactory: DefineTypeFactoryInterfaceRequired<{}, {}>;
//# sourceMappingURL=factory.d.ts.map