// MEMO: The tests for this module are covered by `e2e/*.e2e.ts`.
import { connectionFromArray } from 'graphql-relay';
import { resolveFields } from './field-resolver.js';
import { getSequenceCounter, resetSequence } from './sequence.js';
export function defineTypeFactoryInternal(transientFieldNames, { defaultFields: defaultFieldsResolver, traits, }, seqKey = {}) {
    const getSeq = () => getSequenceCounter(seqKey);
    return {
        async build(inputFieldsResolver) {
            const seq = getSeq();
            return resolveFields(transientFieldNames, seq, defaultFieldsResolver, inputFieldsResolver ?? {});
        },
        async buildList(count, inputFieldsResolver) {
            const array = [];
            for (let i = 0; i < count; i++) {
                if (inputFieldsResolver) {
                    // eslint-disable-next-line no-await-in-loop, @typescript-eslint/no-explicit-any
                    array.push((await this.build(inputFieldsResolver)));
                }
                else {
                    // eslint-disable-next-line no-await-in-loop, @typescript-eslint/no-explicit-any
                    array.push((await this.build()));
                }
            }
            return array;
        },
        async buildConnection(count, connectionArgs, inputFieldsResolver) {
            const list = inputFieldsResolver ? await this.buildList(count, inputFieldsResolver) : await this.buildList(count);
            return connectionFromArray(list, connectionArgs);
        },
        use(traitName) {
            const trait = traits?.[traitName];
            if (!trait)
                throw new Error(`Trait ("${String(traitName)}") is not defined.`);
            // @ts-expect-error -- Use @ts-expect-error as it is impossible to match types.
            return defineTypeFactoryInternal(transientFieldNames, {
                defaultFields: { ...defaultFieldsResolver, ...trait.defaultFields },
                traits,
            }, seqKey);
        },
        resetSequence() {
            resetSequence(seqKey);
        },
    };
}
class DefineTypeFactory extends Function {
    _defaultTransientFields;
    constructor(defaultTransientFields) {
        super();
        this._defaultTransientFields = defaultTransientFields;
        // ref: https://gist.github.com/arccoza/50fe61c8430fc97a463bf6b8960776ce
        // eslint-disable-next-line no-constructor-return
        return new Proxy(this, {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            apply: (target, _thisArg, [options]) => target._apply(options),
        });
    }
    _apply(options) {
        return defineTypeFactoryInternal(Object.keys(this._defaultTransientFields), {
            ...options,
            defaultFields: { ...this._defaultTransientFields, ...options.defaultFields },
        });
    }
    withTransientFields(defaultTransientFields) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return new DefineTypeFactory({ ...this._defaultTransientFields, ...defaultTransientFields });
    }
    withAdditionalFields() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return this;
    }
}
export const defineTypeFactory = new DefineTypeFactory({});
//# sourceMappingURL=factory.js.map