import { DeepReadonly, Merge } from './util.js';
export type FieldResolverOptions<TypeWithTransientFields> = {
    seq: number;
    get: <FieldName extends keyof TypeWithTransientFields>(fieldName: FieldName) => Promise<DeepReadonly<TypeWithTransientFields[FieldName]> | undefined>;
};
export declare class Dynamic<TypeWithTransientFields, Field> {
    private readonly factory;
    constructor(factory: (options: FieldResolverOptions<TypeWithTransientFields>) => Field | Promise<Field>);
    get(options: FieldResolverOptions<TypeWithTransientFields>): Promise<Field>;
}
/** Wrapper to delay field generation until needed. */
export declare function dynamic<TypeWithTransientFields, Field>(factory: (options: FieldResolverOptions<TypeWithTransientFields>) => Field | Promise<Field>): Dynamic<TypeWithTransientFields, Field>;
export type FieldResolver<TypeWithTransientFields, Field> = Field | Dynamic<TypeWithTransientFields, Field>;
/** The type of `defaultFields` or `inputFields` option. */
export type FieldsResolver<TypeWithTransientFields> = {
    [FieldName in keyof TypeWithTransientFields]?: FieldResolver<TypeWithTransientFields, DeepReadonly<TypeWithTransientFields[FieldName]>>;
};
export type ResolvedField<T extends FieldResolver<unknown, unknown>> = T extends FieldResolver<infer _, infer R> ? R : never;
/** Convert `{ a: number, b: () => number, c: () => Promise<number> }` into `{ a: number, b: number, c: number }`. */
export type ResolvedFields<FieldsResolver extends Record<string, FieldResolver<unknown, unknown>>> = {
    [FieldName in keyof FieldsResolver]: ResolvedField<FieldsResolver[FieldName]>;
};
export declare function resolveFields<Type extends Record<string, unknown>, TransientFields extends Record<string, unknown>, _DefaultFieldsResolver extends FieldsResolver<Type & TransientFields>, _InputFieldsResolver extends FieldsResolver<Type & TransientFields>>(transientFieldNames: (keyof TransientFields)[], seq: number, defaultFieldsResolver: _DefaultFieldsResolver, inputFieldsResolver: _InputFieldsResolver): Promise<Omit<Merge<ResolvedFields<_DefaultFieldsResolver>, ResolvedFields<_InputFieldsResolver>>, keyof TransientFields>>;
//# sourceMappingURL=field-resolver.d.ts.map