export class Dynamic {
    factory;
    constructor(factory) {
        this.factory = factory;
    }
    async get(options) {
        return this.factory(options);
    }
}
/** Wrapper to delay field generation until needed. */
export function dynamic(factory) {
    return new Dynamic(factory);
}
export async function resolveFields(transientFieldNames, seq, defaultFieldsResolver, inputFieldsResolver) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Use any type as it is impossible to match types.
    const fields = {};
    async function resolveField(options, fieldResolver) {
        if (fieldResolver instanceof Dynamic) {
            return fieldResolver.get(options);
        }
        else {
            return fieldResolver;
        }
    }
    async function resolveFieldAndUpdateCache(fieldName) {
        if (fieldName in fields)
            return fields[fieldName];
        let fieldResolver;
        if (fieldName in inputFieldsResolver) {
            fieldResolver = inputFieldsResolver[fieldName];
        }
        else if (fieldName in defaultFieldsResolver) {
            fieldResolver = defaultFieldsResolver[fieldName];
        }
        else {
            return undefined;
        }
        // eslint-disable-next-line require-atomic-updates
        fields[fieldName] = await resolveField(options, fieldResolver);
        return fields[fieldName];
    }
    const options = {
        seq,
        get: resolveFieldAndUpdateCache,
    };
    for (const fieldName of [...Object.keys(defaultFieldsResolver), ...Object.keys(inputFieldsResolver)]) {
        // eslint-disable-next-line no-await-in-loop
        await resolveFieldAndUpdateCache(fieldName);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Use any type as it is impossible to match types.
    return Object.fromEntries(Object.entries(fields).filter(([key]) => !transientFieldNames.includes(key)));
}
//# sourceMappingURL=field-resolver.js.map